@@ -55,6 +55,7 @@
 #include "smp.h"
 #include "svm.h"
 #include "svm_init.h"
+#include "tty.h"
 #include "vcpu.h"
 #include "vm.h"
 #include "vmmcall.h"
@@ -72,7 +73,7 @@ print_boot_msg (void)
 	printf ("Starting VMM (x86_32)...\n");
 #endif
 	printf ("Copyright (c) 2007, 2008 University of Tsukuba\n");
-	printf ("Copyright (c) 2010-2012 Yuichi Watanabe\n");
+	printf ("Copyright (c) 2010-2013 Yuichi Watanabe\n");
 	printf ("All rights reserved.\n");
 }
 
@@ -204,20 +205,14 @@ initregs (bool vbsp, u8 bios_boot_drive)
 				printf("Boot from 0x%x drive.\n",
 				       bios_boot_drive);
 			}
-			if (param_get_str(VM0_NAME ".boot_parttion",
+			if (param_get_str(VM0_NAME ".boot_partition",
 					  0, NULL, 0) == VMMERR_SUCCESS) {
 				current->vmctl.write_general_reg(
 					GENERAL_REG_RSI, 1);
 				printf("Use loader of active partition\n");
 			}
 		}
-	} else if (vbsp) {
-		/*
-		 * A vm except vm0 boots from guest BIOS.
-		 */
-		current->vmctl.write_realmode_seg (SREG_CS, 0xf000);
-		current->vmctl.write_ip (0xfff0);
-	} else {
+	} else if (!vbsp) {
 		apic_enter_wait_for_sipi_state();
 	}
 }
@@ -326,7 +321,7 @@ ap_main (void)
 		if (get_apic_id() == shell_apic_id) {
 			int i;
 
-			for (i = 0; i < 12; i++) {
+			for (i = 0; i < 14; i++) {
 				sync_all_processors();
 			}
 			for (;;) {
@@ -336,7 +331,17 @@ ap_main (void)
 	}
 
 	setup_vm();
-	sync_all_processors();
+	sync_all_processors(); /* 12 */
+
+	if (current->vbsp) {
+		if (!current->vm->iommu_enabled) {
+			panic("IOMMU is not enabled. VMM can't start more than one VMs");
+		}
+	}
+
+	sync_all_processors(); /* 13 */
+	sync_all_processors(); /* 14 */
+
 	run_vm();
 
 	panic ("VM stopped.");
@@ -356,10 +361,15 @@ bsp_main (void)
 	pci_dump_resources();
 #endif
 	call_initfunc ("start");
+
+	sync_all_processors(); /* 12 */
+	sync_all_processors(); /* 13 */
+
 	print_startvm_msg();
 	sync_cursor_pos ();
 	vramwrite_exit_global();
-	sync_all_processors();
+
+	sync_all_processors(); /* 14 */
 
 	if (param_get_u32("shell", 0, &shell_apic_id) == VMMERR_SUCCESS) {
 		if (get_apic_id() == shell_apic_id) {
@@ -376,6 +386,8 @@ bsp_main (void)
 asmlinkage void
 vmm_start (struct multiboot_info *mi_arg)
 {
+	tty_init_global();
+	print_boot_msg();
 	memcpy (&multiboot_info, mi_arg, sizeof (struct multiboot_info));
 	initfunc_init ();
 	call_initfunc ("global");
@@ -385,6 +397,5 @@ vmm_start (struct multiboot_info *mi_arg)
 }
 
 INITFUNC ("pcpu3", virtualization_init_pcpu);
-INITFUNC ("global1", print_boot_msg);
 INITFUNC ("global3", copy_cmdline);
 INITFUNC ("global3", copy_gbios);
